/*********************************************************************/
/*                                                                   */
/*   Program Name :  signalcheck                                     */
/*                                                                   */
/*      Version:  0.00  2002-10-04                                   */
/*                0.10  2002-10-04                                   */
/*                1.13  2005-09-28   support linux2                  */
/*                2.00  2005-12-20  support VSSP32                   */
/*                2.10  2007-02-23  revised to use common library    */
/*                2.11  2007-08-23  add FPGA version display         */
/*                2.12  2008-02-19  options '--help' and '--version' */
/*                                  wereadded                        */
/*                2.20  2016-05-18  support VSSP64                   */
/*                                                                   */
/*      Copyright (c) 2002-2016  T.Kondo/NICT All Right Reserved     */
/*                                                                   */
/*********************************************************************/
/*
/*
     Signalcheck checks whether 10MHz and 1PPS is supplied to a K5/VSSP or K5/VSSP32
     or not. FPGA version is also displayed in case of VSSP32 of which Version after 2007-08-17
     
        How to use
             signalcheck


*/
#define PROGRAM_VERSION "Ver. 2016-05-18"
#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

char dev[30] ;  // IP-VLBI BOARD control variable     2016-05-13
//int g_model=2;            // VSSP model   0: VSSP 1: VSSP32  2: VSSP64


int main(int argc, char **argv)
{
    //char dev[] = "/dev/tds0";
    //char dev[] = DEV_FILE;
    int fd_in,rv;
    int fpga_version;
    unsigned int stat;
    unsigned int ref, pps, rhz;
#ifndef _VSSP
    struct tdsio tdsio;    // 2007-08-23
#endif

    if(argc == 2 && strncmp(*(argv+1),"--help",6)==0 ){ // 2008-02-19
       printf("\n signalcheck (%s)\n",PROGRAM_VERSION);
       printf("     compiled for K5/%s sampler\n",VSSP_MODEL_NAME);
       printf("\n");
       printf("Signalcheck checks whether 10MHz and 1PPS is supplied to a K5/%s\n",VSSP_MODEL_NAME);
       printf("or not. FPGA version is also displayed in case of VSSP32 or VSSP64 of which Version after 2007-08-17\n");
       
       printf("\n");
       printf("Usage: signalcheck \n");
       printf("\n");
       exit(0);
    } 
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
       printf("\n signalcheck (%s)\n",PROGRAM_VERSION);
       printf("     compiled for K5/%s sampler\n",VSSP_MODEL_NAME);
       printf("\n");
       exit(0);
    } 
    
    
    
    printf("\n signalcheck (%s)\n",PROGRAM_VERSION);


     // get device file name
    g_vssp_model=VSSP_MODEL;
    get_vssp_drv_name(g_vssp_model, g_dev_file);    // get device file name for VSSP series
   
    printf("  Device file => %s\n",g_dev_file);
    printf("\n  K5/%s board Reference and 1PPS signal checking..\n\n",VSSP_MODEL_NAME);
    fd_in = open(g_dev_file, O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }

    /* board reference signal check */
    stat=board_ref_check(fd_in);
    if((stat & 0x10) == 0x10){ // this is VSSP32
        if((stat & 1) != 1){
            printf("   Warning! 10MHz or 5MHz signal is not supplied. \n");
        } else {
            if((stat & 4) == 4){
                printf("   Reference (5MHz) signal is OK!");
            } else {
                printf("   Reference (10MHz) signal is OK!");
            }
            if((stat & 8) == 8){
                printf("   PLL locked!\n");
            } else {
                printf("   Warning! PLL unlocked!\n");
            }
        }
    } else {   // this is VSSP
        if((stat & 1) != 1){
            printf("   Warning! 10MHz signal is not supplied. \n");
        } else {
            printf("   10MHz signal is OK!\n");
        }
    }
    if((stat & 2) != 2){
        printf("   Warning! 1PPS signal is not supplied. \n");
    } else {
        printf("   1PPS  signal is OK!\n");
    }
    if((stat & 3) == 0 || (stat & 3) == 2){
       printf("   Check signal connection and try again!\n");
       goto end;
    }
end:
    close(fd_in);

    /* get FPGA version */
    fpga_version=get_fpga_version();
    if(fpga_version > 0){
        printf("\n   %s FPGA Version = %d.%02d\n" ,VSSP_MODEL_NAME,fpga_version/100, fpga_version % 100);
    }
    printf("\n");

}
