/**********************************************************************/
/*                                                                    */
/*   Program Name :  timesetpc                                        */
/*                                                                    */
/*      Version:  1.00  2007-02-07                                    */
/*                1.10  2007-02-23  revised to use common library     */
/*                1.11  2008-02-19  options '--help' and '--version'  */
/*                                  were added                        */
/*                1.20  2016-05-19  support VSSP64                    */
/*                                                                    */
/*        K5/VSSP or K5/VSSP32 board time set using PC time           */
/*           (This is for testing purpose)                            */
/*                                                                    */
/*      Copyright (c) 2007-2016 T.Kondo/NICT All Right Reserved       */
/*                                                                    */
/**********************************************************************/
/*
/*
     K5/VSSP or K5/VSSP32 Time Set using PC time
        How to use:
          timesetpc 1

              Time will be diplayed for 10 sec after succession of time set
             

*/
#define TIMESETPC_VERSION "Ver. 2016-05-19"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

int main(int argc, char **argv)
{
    //char dev[] = DEV_FILE; 
    char dev[30]; 
    int fd_in,rv, sig;
    unsigned int time,day,sec; 
    unsigned int secold, stat;
    int  yyyy, mm, dd;
    int ddd;
    int  h, m, s;  
    int  y3; // 7 bits year  (0-127)
    float fsec,tofset;
    int counter;
    int i;
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char logdir[60];               // logfile directory
    char hostname[20];             // host name
    char cbuf[120];
    
    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
        printf("\n");
        printf("timesetpc %s\n", TIMESETPC_VERSION);
        printf("      compiled for K5/%s sampler\n",VSSP_MODEL_NAME);
        printf("\n");
        printf("  Usage: timesetpc 1\n");
        printf("    K5/%s sampler time will be set using PC time\n",VSSP_MODEL_NAME);
        printf("    This is just for a board testing purpose.\n");
        printf("    Do not use in an actual VLBI observation!.\n\n");
        exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
        printf("\n");
        printf("timesetpc %s\n", TIMESETPC_VERSION);
        printf("      compiled for K5/%s sampler\n",VSSP_MODEL_NAME);
        printf("\n");
        exit(0);
    }
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 
    if(atoi(*(argv + 1 )) != 1) exit(0);
   
    // get device file name
    g_vssp_model=VSSP_MODEL;
    get_vssp_drv_name(g_vssp_model, g_dev_file);    // get device file name for VSSP series
    //printf("Device file => %s\n",g_dev_file);
    strcpy(dev,g_dev_file);
    printf("Device file => %s\n",dev);

    /* time get from PC */
    
    tofset=0.0;
    getsystimef(&yyyy, &mm, &dd, &h, &m, &s, &fsec, &tofset);

    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);

    sprintf(cbuf,"timesetpc(%s) %d %d %d %d %d %d",TIMESETPC_VERSION,yyyy,mm,dd,h,m,s);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    fd_in = open(dev, O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        sprintf(cbuf,"device (%s) open() error",dev);
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    //printf("Device open successd No.[%d]\n",fd_in);
    
    /* board reference signal check */
    printf("\n");
    if((sig=board_ref_check2(fd_in))==-1) {
        close(fd_in);
        goto end;
    }
    
    /* time set */
    //printf("%d,%d,%d,%d,%d,%d\n",yyyy,mm,dd,h,m,s);  // debug

    //day = y2k(yyyy, mm, dd);
    md2ddd(yyyy, &ddd, mm, dd);
    day = (unsigned int)ddd;
    sec = ((h * 60) + m) * 60 + s;
    time = TDS_MAKE_TIME(day,sec);
    
    printf("   timesetpc(%s) %d %d %d %d %d %d\n",TIMESETPC_VERSION,yyyy,mm,dd,h,m,s);
    printf("   Set time  => Total day : %5d    Seconds from 0h : %5d \n",day,sec);
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    //year set
    y3=yyyy-2000;
    rv = ioctl(fd_in, TDSIO_SET_YEAR, &y3);  
    if (rv == -1) {
        perror("ioctl() year set failed ");
        sprintf(cbuf,"ioctl() year set failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
#endif

    //date and seconds set to sampler
    rv = ioctl(fd_in, TDSIO_SET_TIME, &time);  
    if (rv == -1) {
        perror("ioctl() time set failed ");
        sprintf(cbuf,"ioctl() time set failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    //printf("time set ok\n");

    //sync to 1PPS
    if(sig==0){
        rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
        if (rv == -1) {
            perror("ioctl() 1pps sync failed ");
            sprintf(cbuf,"ioctl() 1pps sync failed");
            if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
            close(fd_in);
            return -1;
        }
        printf("   1pps sync ok\n");
    }

    time = 0; 
    secold=90000;
    counter=0;
    close(fd_in);
	
    
    // log out
    if(sig==0){
       sprintf(cbuf,"timesetpc (board time set using PC time is  OK)");
    } else {
       sprintf(cbuf,"timesetpc (board time set using PC time is OK but 1PPS was not supplied! (no 1PPS sync made))");
    }
    if(LogWrite2(logdir,hostname,1,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");

    // display time for given time seconds
    printf("\n");
    i=board_time_disp(10);
    
    printf("\n\n*** time set K5/%s using PC time is successed ",VSSP_MODEL_NAME);
    if(sig !=0) printf("(but no 1PPS signal)");
    printf(" ***\n\n");
end:
    return 0;
}
